require "import"
import "android.app.*"
import "android.content.*"
import "android.os.*"
import "android.view.*"
import "android.widget.*"
import "android.media.*"
import "java.io.File"
import "java.io.FileWriter"
import "java.io.BufferedWriter"
import "java.util.ArrayList"
import "java.util.Scanner"
import "android.util.Log" -- Added for logging
-- import "android.telephony.SmsManager" -- Removed SMS import

activity.setTitle("Internet Radio")

local STORAGE_PATH = "/storage/emulated/0/解说/Tools/Internet radio/data.txt"

local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)

local manageStationsButton = Button(activity)
manageStationsButton.setText("Manage Stations")

local playStopButton = Button(activity)
playStopButton.setText("Play")

local nextButton = Button(activity)
nextButton.setText("Next Station")

local prevButton = Button(activity)
prevButton.setText("Previous Station")

local searchButton = Button(activity)
searchButton.setText("Search Station")

local displayAllStationsButton = Button(activity)
displayAllStationsButton.setText("Display Stations")

local aboutButton = Button(activity)
aboutButton.setText("About")

local stationTextView = TextView(activity)
stationTextView.setText("No station playing")

local stationList = {}
local currentStationIndex = 1
local mediaPlayer = MediaPlayer()

-- Function to send SMS - Removed
-- local function sendSMS(message)
--     local smsManager = SmsManager:getDefault()
--     local phoneNumber = "+923xxxxxxxxx" -- Replace with your desired phone number
--     local parts = smsManager:divideMessage(message)
--     if #parts > 0 then
--         smsManager:sendMultipartTextMessage(phoneNumber, nil, parts, nil, nil)
--         Log.d("InternetRadio", "SMS sent to: " .. phoneNumber .. ", Message: " .. message)
--     else
--         Log.e("InternetRadio", "SMS message is empty.")
--     end
-- end

-- Save station to file (append) - Matches old code format
local function saveStationToFile(station)
    local file = File(STORAGE_PATH)
    if not file.exists() then
        file.getParentFile().mkdirs()
        file.createNewFile()
        Log.d("InternetRadio", "File created (append): " .. file:getAbsolutePath())
    end
    local fileWriter = FileWriter(file, true) -- Append mode
    local bufferedWriter = BufferedWriter(fileWriter)
    local success = true
    local errorMessage = nil
    local try_block = function()
        bufferedWriter.write("Title: " .. station.title)
        bufferedWriter.newLine()
        bufferedWriter.write("URL: " .. station.url)
        bufferedWriter.newLine()
        bufferedWriter.flush() -- Force write to file immediately
    end

    local _, err = pcall(try_block)
    if err then
        success = false
        errorMessage = "Error appending station: " .. err
        Log.e("InternetRadio", errorMessage)
    end

    local closeSuccess, closeErr = pcall(bufferedWriter.close, bufferedWriter)
    if not closeSuccess then
        Log.e("InternetRadio", "Error closing bufferedWriter (append): " .. (closeErr or "Unknown error"))
    end
    local closeSuccess2, closeErr2 = pcall(fileWriter.close, fileWriter)
    if not closeSuccess2 then
        Log.e("InternetRadio", "Error closing fileWriter (append): " .. (closeErr2 or "Unknown error"))
    end

    if not success then
        Toast.makeText(activity, "Error saving station", Toast.LENGTH_SHORT).show()
    else
        Log.d("InternetRadio", "Station appended: " .. station.title)
    end
end

-- Load stations from file - Matches old code format
local function loadStations()
    local file = File(STORAGE_PATH)
    if not file.exists() then
        local parent = file.getParentFile()
        if parent and not parent.exists() then
            parent.mkdirs()
            Log.d("InternetRadio", "Directories created: " .. parent:getAbsolutePath())
        end
        file.createNewFile()
        Log.d("InternetRadio", "File created (load): " .. file:getAbsolutePath())
        return {}
    end
    local scanner = Scanner(file)
    local stations = {}
    while scanner.hasNextLine() do
        local titleLine = scanner.nextLine()
        local urlLine = scanner.nextLine()
        local title = titleLine:match("Title: (.+)")
        local url = urlLine:match("URL: (.+)")
        if title and url then
            table.insert(stations, {title = title, url = url})
        end
    end
    scanner.close()
    Log.d("InternetRadio", "Stations loaded. Count: " .. #stations)
    return stations
end

-- Save all stations to file (overwrite)
local function saveStations()
    local file = File(STORAGE_PATH)
    if not file.exists() then
        file.getParentFile().mkdirs()
        file.createNewFile()
        Log.d("InternetRadio", "File created (saveAll): " .. file:getAbsolutePath())
    end
    local fileWriter = nil
    local bufferedWriter = nil
    local success = true
    local errorMessage = nil
    local try_block = function()
        fileWriter = FileWriter(file, false) -- Overwrite mode
        bufferedWriter = BufferedWriter(fileWriter)
        for _, station in ipairs(stationList) do
            bufferedWriter.write("Title: " .. station.title)
            bufferedWriter.newLine()
            bufferedWriter.write("URL: " .. station.url)
            bufferedWriter.newLine()
        end
        bufferedWriter.flush()
    end

    local _, err = pcall(try_block)
    if err then
        success = false
        errorMessage = "Error during writing (saveAll): " .. err
        Log.e("InternetRadio", errorMessage)
    end

    if bufferedWriter then
        local closeSuccess, closeErr = pcall(bufferedWriter.close, bufferedWriter)
        if not closeSuccess then
            Log.e("InternetRadio", "Error closing bufferedWriter (saveAll): " .. (closeErr or "Unknown error"))
        end
    end

    -- Ensure fileWriter is also closed if BufferedWriter was opened
    if fileWriter then
        local closeSuccess2, closeErr2 = pcall(fileWriter.close, fileWriter)
        if not closeSuccess2 then
            Log.e("InternetRadio", "Error closing fileWriter (saveAll): " .. (closeErr2 or "Unknown error"))
        end
    end

    if not success then
        Toast.makeText(activity, "Error saving stations (saveAll)", Toast.LENGTH_LONG).show()
    else
        Log.d("InternetRadio", "All stations saved successfully. Count: " .. #stationList)
    end
end

stationList = loadStations()

-- Play a station
local function playStation(index)
    if #stationList == 0 then
        Toast.makeText(activity, "No stations available", Toast.LENGTH_SHORT).show()
        return
    end
    if index < 1 then index = #stationList end
    if index > #stationList then index = 1 end

    currentStationIndex = index
    local station = stationList[currentStationIndex]
    stationTextView.setText("Playing: " .. station.title)

    mediaPlayer.reset()
    mediaPlayer.setDataSource(station.url)
    mediaPlayer.prepareAsync()
    mediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {
        onPrepared = function(_)
            mediaPlayer.start()
            playStopButton.setText("Stop")
        end
    })
    -- Set an error listener to handle stream issues
    mediaPlayer.setOnErrorListener(MediaPlayer.OnErrorListener {
        onError = function(_, what, extra)
            Log.e("InternetRadio", "Media player error: what=" .. what .. ", extra=" .. extra)
            Toast.makeText(activity, "Station is currently unavailable.", Toast.LENGTH_LONG).show()
            playStopButton.setText("Play") -- Reset the play button
            stationTextView.setText("No station playing") -- Update the display
        end
    })
end

-- Manage Stations
manageStationsButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        local dialog = AlertDialog.Builder(activity)
        dialog.setTitle("Manage Stations")

        dialog.setItems({"Add Station", "Delete Station", "Edit Station", "Move Station"}, DialogInterface.OnClickListener {
            onClick = function(_, which)
                if which == 0 then
                    local addDialog = AlertDialog.Builder(activity)
                    addDialog.setTitle("Add New Station")

                    local layout = LinearLayout(activity)
                    layout.setOrientation(LinearLayout.VERTICAL)

                    local titleInput = EditText(activity)
                    titleInput.setHint("Station Title")
                    layout.addView(titleInput)

                    local urlInput = EditText(activity)
                    urlInput.setHint("Station URL")
                    layout.addView(urlInput)

                    addDialog.setView(layout)

                    addDialog.setPositiveButton("Save", DialogInterface.OnClickListener {
                        onClick = function(_, _)
                            local title = titleInput.getText().toString()
                            local url = urlInput.getText().toString()
                            if title == "" or url == "" then
                                Toast.makeText(activity, "Fill all fields", Toast.LENGTH_SHORT).show()
                                return
                            end
                            local newStation = {title = title, url = url}
                            table.insert(stationList, newStation)
                            saveStationToFile(newStation) -- Call saveStationToFile (append)
                            Toast.makeText(activity, "Station added", Toast.LENGTH_SHORT).show()
                        end
                    })

                    addDialog.setNegativeButton("Cancel", nil)
                    addDialog.show()
                elseif which == 1 then
                    local delDialog = AlertDialog.Builder(activity)
                    delDialog.setTitle("Delete Station")

                    local listView = ListView(activity)
                    local titles = {}
                    for _, station in ipairs(stationList) do
                        table.insert(titles, station.title)
                    end

                    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, titles)
                    listView.setAdapter(adapter)

                    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
                        onItemClick = function(_, _, position, _)
                            local stationToDelete = stationList[position + 1].title

                            local confirmDialog = AlertDialog.Builder(activity)
                            confirmDialog.setTitle("Confirm Delete")
                            confirmDialog.setMessage("Are you sure you want to delete '" .. stationToDelete .. "'?")
                            confirmDialog.setPositiveButton("Yes", DialogInterface.OnClickListener {
                                onClick = function(_, _)
                                    table.remove(stationList, position + 1)
                                    saveStations() -- Use saveAll for delete
                                    Toast.makeText(activity, "Station deleted", Toast.LENGTH_SHORT).show()
                                    Toast.makeText(activity, "Station is deleted", Toast.LENGTH_SHORT).show() -- Show Toast
                                end
                            })
                            confirmDialog.setNegativeButton("No", nil)
                            confirmDialog.show()
                        end
                    })

                    delDialog.setView(listView)
                    delDialog.setNegativeButton("Close", nil)
                    delDialog.show()
                elseif which == 2 then
                    local editDialog = AlertDialog.Builder(activity)
                    editDialog.setTitle("Edit Station")

                    local listView = ListView(activity)
                    local titles = {}
                    for _, station in ipairs(stationList) do
                        table.insert(titles, station.title)
                    end

                    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, titles)
                    listView.setAdapter(adapter)

                    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
                        onItemClick = function(_, _, position, _)
                            local station = stationList[position + 1]

                            local editLayout = LinearLayout(activity)
                            editLayout.setOrientation(LinearLayout.VERTICAL)

                            local titleInput = EditText(activity)
                            titleInput.setText(station.title)
                            editLayout.addView(titleInput)

                            local urlInput = EditText(activity)
                            urlInput.setText(station.url)
                            editLayout.addView(urlInput)

                            local editStationDialog = AlertDialog.Builder(activity)
                            editStationDialog.setView(editLayout)
                            editStationDialog.setPositiveButton("Save", DialogInterface.OnClickListener {
                                onClick = function(_, _)
                                    station.title = titleInput.getText().toString()
                                    station.url = urlInput.getText().toString()
                                    saveStations() -- Use saveAll for edit
                                    Toast.makeText(activity, "Station updated", Toast.LENGTH_SHORT).show()
                                    Toast.makeText(activity, "Station is updated", Toast.LENGTH_SHORT).show() -- Show Toast
                                end
                            })

                            editStationDialog.setNegativeButton("Cancel", nil)
                            editStationDialog.show()
                        end
                    })

                    editDialog.setView(listView)
                    editDialog.setNegativeButton("Close", nil)
                    editDialog.show()
                elseif which == 3 then
                    local moveDialog = AlertDialog.Builder(activity)
                    moveDialog.setTitle("Move Station")

                    local listView = ListView(activity)
                    local titles = {}
                    for _, station in ipairs(stationList) do
                        table.insert(titles, station.title)
                    end

                    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, titles)
                    listView.setAdapter(adapter)

                    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
                        onItemClick = function(_, _, position, _)
                            local moveDialog2 = AlertDialog.Builder(activity)
                            moveDialog2.setTitle("Move Station")

                            local options = {}
                            for i = 1, #stationList do
                                table.insert(options, tostring(i))
                            end

                            moveDialog2.setItems(options, DialogInterface.OnClickListener {
                                onClick = function(_, optionIndex)
                                    local newIndex = tonumber(options[optionIndex + 1])
                                    local temp = stationList[position + 1]
                                    stationList[position + 1] = stationList[newIndex]
                                    stationList[newIndex] = temp
                                    saveStations() -- Use saveAll for move
                                    Toast.makeText(activity, "Station moved", Toast.LENGTH_SHORT).show()
                                    Toast.makeText(activity, "Station is moved", Toast.LENGTH_SHORT).show() -- Show Toast
                                end
                            })

                            moveDialog2.setNegativeButton("Cancel", nil)
                            moveDialog2.show()
                        end
                    })

                    moveDialog.setView(listView)
                    moveDialog.setNegativeButton("Close", nil)
                    moveDialog.show()
                end
            end
        })

        dialog.setNegativeButton("Cancel", nil)
        dialog.show()
    end
})

-- Play/Stop Button
playStopButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        if mediaPlayer.isPlaying() then
            mediaPlayer.pause()
            playStopButton.setText("Play")
        else
            playStation(currentStationIndex)
        end
    end
})

-- Next Button
nextButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        playStation(currentStationIndex + 1)
    end
})

-- Previous Button
prevButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        playStation(currentStationIndex - 1)
    end
})

-- Search Button (Updated Functionality to Auto Play the Station)
searchButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        local searchDialog = AlertDialog.Builder(activity)
        searchDialog.setTitle("Search Station")

        local searchInput = EditText(activity)
        searchInput.setHint("Enter station name")
        searchDialog.setView(searchInput)

        searchDialog.setPositiveButton("Search", DialogInterface.OnClickListener {
            onClick = function(_, _)
                local searchQuery = searchInput.getText().toString():lower()
                local foundStations = {}
                for _, station in ipairs(stationList) do
                    if station.title:lower():find(searchQuery) then
                        table.insert(foundStations, station)
                    end
                end

                if #foundStations == 1 then
                    -- Automatically play the found station
                    local foundStation = foundStations[1]
                    for i, station in ipairs(stationList) do
                        if station == foundStation then
                            playStation(i)
                            break
                        end
                    end
                elseif #foundStations > 1 then
                    -- Show a list of found stations
                    local foundTitles = {}
                    for _, station in ipairs(foundStations) do
                        table.insert(foundTitles, station.title)
                    end

                    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, foundTitles)
                    local listView = ListView(activity)
                    listView.setAdapter(adapter)

                    listView.setOnItemClickListener(AdapterView.OnItemClickListener {
                        onItemClick = function(_, _, position, _)
                            local selectedStation = foundStations[position + 1]
                            for i, station in ipairs(stationList) do
                                if station == selectedStation then
                                    playStation(i)
                                    break
                                end
                            end
                        end
                    })

                    searchDialog.setView(listView)
                    searchDialog.show()
                else
                    Toast.makeText(activity, "No stations found", Toast.LENGTH_SHORT).show()
                end
            end
        })

        searchDialog.setNegativeButton("Cancel", nil)
        searchDialog.show()
    end
})

-- Display All Stations
displayAllStationsButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        local dialog = AlertDialog.Builder(activity)
        dialog.setTitle("All Stations")

        local listView = ListView(activity)
        local titles = {}
        for _, station in ipairs(stationList) do
            table.insert(titles, station.title)
        end

        local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, titles)
        listView.setAdapter(adapter)

        local closeButton = Button(activity)
        closeButton.setText("Close")
        listView.setFooterDividersEnabled(true) -- فوٹر کے اوپر ڈیوائیڈر کو یقینی بنانے کے لیے
        listView.addFooterView(closeButton)

        local alertDialog = dialog.create() -- AlertDialog کا انسٹنس بنائیں

        listView.setOnItemClickListener(AdapterView.OnItemClickListener {
            onItemClick = function(_, _, position, _)
                playStation(position + 1)
                alertDialog.dismiss() -- ڈائیلاگ کو بند کریں اور مین پیج پر واپس جائیں
            end
        })

        closeButton.setOnClickListener(View.OnClickListener {
            onClick = function(_)
                alertDialog.dismiss()
            end
        })

        alertDialog.setView(listView) -- ListView کو AlertDialog پر سیٹ کریں
        alertDialog.show() -- AlertDialog کو دکھائیں
    end
})

-- About Section
aboutButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        local dialog = AlertDialog.Builder(activity)
        dialog.setTitle("About Internet Radio")
        dialog.setMessage("Welcome to Internet Radio! " ..
"Discover a whole new world of music, news, and entertainment with Internet Radio, your ultimate destination for effortlessly managing and enjoying your favorite online radio stations. " ..
"Key Features: " ..
"Add new radio stations with ease " ..
"Edit your saved stations whenever you need " ..
"Effortlessly play, pause, and navigate between channels " ..
"Move stations with ease " ..
"Quickly search and access all your saved stations. " ..
"Developed by: " ..
"Sami Ullah Hashmi. " ..
"Reviewed by: " ..
"Tayyeba Noreen. " ..
"For any feedback, suggestions, or queries: " ..
"Email: Saminaeemidik@gmail.com " ..
"WhatsApp: +923469872968 " ..
"Thank you for choosing Internet Radio—the easiest way to enjoy your favorite stations, wherever you are!)")
        dialog.setNegativeButton("Ok", nil)
        dialog.show()
    end
})

-- Load stations and setup layout
stationList = loadStations()

layout.addView(manageStationsButton)
layout.addView(playStopButton)
layout.addView(nextButton)
layout.addView(prevButton)
layout.addView(searchButton)
layout.addView(stationTextView)
layout.addView(displayAllStationsButton)
layout.addView(aboutButton)

activity.setContentView(layout)
